#region ... Copyright  2005 Roland Weigelt ...
/*
This software is provided 'as-is', without any expressed or
implied warranty. In no event will the author be held liable
for any damages arising from the use of this software.

Permission is granted to anyone to use this software for any
purpose, including commercial applications, and to alter it and
redistribute it freely, subject to the following restrictions:

1.	The origin of this software must not be misrepresented; you
	must not claim that you wrote the original software. If you
	use this software in a product, an acknowledgment in the
	product documentation is required.
	
2.	Altered source versions must be plainly marked as such, and
	must not be misrepresented as being the original software.
	
3.	This notice may not be removed or altered from any source
	or altered source distribution.

Copyright  2005 Roland Weigelt (info@roland-weigelt.de)
*/
#endregion

using System;
using System.Collections;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Configuration;
using System.Data;
using System.Diagnostics;
using System.ServiceProcess;
using System.Text;

namespace Weigelt.LastBoxStanding
{
	/// <summary>
	/// Represents the running service.
	/// </summary>
	public class TheService : System.ServiceProcess.ServiceBase
	{
		private DateTime m_dtmTimerStopped; // Contains either the time when the timer was stopped or DateTime.MaxValue when the timer is running; used for detecting spurious timer events.
		private Worker m_objWorker;
		private System.Timers.Timer m_objTimer;
		private System.ComponentModel.Container components = null;

		/// <summary>
		/// Writes a message to the event log.
		/// </summary>
		/// <param name="type">Type of the event log entry.</param>
		/// <param name="message">Message of the event log entry.</param>
		private void WriteEventLogEntry(EventLogEntryType type, string message)
		{
			this.EventLog.WriteEntry(message, type);
		}

		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.m_objTimer = new System.Timers.Timer();
			((System.ComponentModel.ISupportInitialize)(this.m_objTimer)).BeginInit();
			// 
			// m_objTimer
			// 
			this.m_objTimer.Interval = 60000;
			this.m_objTimer.Elapsed += new System.Timers.ElapsedEventHandler(this.HandleTimerElapsed);
			// 
			// TheService
			// 
			this.ServiceName = "LastBoxStanding";
			((System.ComponentModel.ISupportInitialize)(this.m_objTimer)).EndInit();

		}

		private void HandleTimerElapsed(object sender, System.Timers.ElapsedEventArgs e)
		{
			if (e.SignalTime>=m_dtmTimerStopped)
				return; // this is an unwanted timer event.

			m_objWorker.Run();

			if (m_objTimer.AutoReset==false)
			{
				//
				// This is the first time the timer has fired, so now let's set
				// it usual mode
				//
				m_objTimer.Enabled=false;
				m_objTimer.Interval=m_objWorker.Configuration.CheckInterval*60*1000; // CheckInterval is specified in minutes
				m_objTimer.AutoReset=true;
				m_objTimer.Enabled=true;
			}
		}

		#region ... Protected overrides ...

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		/// <summary>
		/// Hook: Called for starting this service.
		/// </summary>
		protected override void OnStart(string[] args)
		{
			m_objWorker.Reset();

			ServiceConfiguration objConfiguration=ConfigurationSettings.GetConfig("lastBoxStanding") as ServiceConfiguration;
			if (objConfiguration==null)
			{
				WriteEventLogEntry(EventLogEntryType.Error, "Missing configuration file.");
			}
			else
			{
				if (objConfiguration.MonitoredMachines.Count==0)
					WriteEventLogEntry(EventLogEntryType.Warning, "No monitored machines specified in configuration.");
				else
				{
					if (objConfiguration.Verbose)
					{
						StringBuilder stbMessage=new StringBuilder("Machines to be monitored:");
						foreach (string strMachine in objConfiguration.MonitoredMachines)
						{
							stbMessage.Append(Environment.NewLine);
							stbMessage.Append(strMachine);
						}
						stbMessage.Append(Environment.NewLine);
						stbMessage.AppendFormat("Check interval: {0}min", objConfiguration.CheckInterval);
						stbMessage.Append(Environment.NewLine);
						stbMessage.AppendFormat("Number of retries: {0}", objConfiguration.CheckRetries);
						WriteEventLogEntry(EventLogEntryType.Information, stbMessage.ToString());
					}
				}

				if ((objConfiguration.ActionWhenLastRemaining==null) ||
					(objConfiguration.ActionWhenLastRemaining.Executable==null) ||
					(objConfiguration.ActionWhenLastRemaining.Executable.Length==0))
				{
					WriteEventLogEntry(EventLogEntryType.Warning, "\"whenLastRemaining\" action not specified in configuration.");
				}

				m_objWorker.Configuration=objConfiguration;
				//
				// We would now need to call the worker's Run() method to avoid a
				// delay in which other machines could be shut down (which is in
				// fact only a problem when trying the service for the first time or
				// when reconfiguring shortly before shutting down the client).
				// The problem is that OnStart is not a place to waste time in and
				// the worker's Run() method can take quite to time to finish.
				// Instead of using a BeginInvoke without an EndInvoke, we'll let
				// the timer fire a single event (before switching to its usual mode)
				//
				m_objTimer.Interval=500; // let the timer fire in 0.5 seconds...
				m_objTimer.AutoReset=false; // ... but only once
				m_dtmTimerStopped=DateTime.MaxValue;
				m_objTimer.Enabled=true;
			}
		}
 
		/// <summary>
		/// Called for stopping this service.
		/// </summary>
		protected override void OnStop()
		{
			m_dtmTimerStopped=DateTime.Now;
			m_objTimer.Enabled=false;
		}
		#endregion

		#region ... Entry point ...

		/// <summary>
		/// The main entry point for the process
		/// </summary>
		static void Main()
		{
			System.ServiceProcess.ServiceBase[] arrServicesToRun=
				new System.ServiceProcess.ServiceBase[] { new TheService() };
			System.ServiceProcess.ServiceBase.Run(arrServicesToRun);
		}

		#endregion

		#region ... Constructor ...

		/// <summary>
		/// Creates a new <see cref="TheService"/> instance.
		/// </summary>
		public TheService()
		{
			InitializeComponent();

			m_objWorker=new Worker(this);
		}

		#endregion
	}
}
