#region ... Copyright  2005 Roland Weigelt ...
/*
This software is provided 'as-is', without any expressed or
implied warranty. In no event will the author be held liable
for any damages arising from the use of this software.

Permission is granted to anyone to use this software for any
purpose, including commercial applications, and to alter it and
redistribute it freely, subject to the following restrictions:

1.	The origin of this software must not be misrepresented; you
	must not claim that you wrote the original software. If you
	use this software in a product, an acknowledgment in the
	product documentation is required.
	
2.	Altered source versions must be plainly marked as such, and
	must not be misrepresented as being the original software.
	
3.	This notice may not be removed or altered from any source
	or altered source distribution.

Copyright  2005 Roland Weigelt (info@roland-weigelt.de)
*/
#endregion

using System;
using System.Net;
using System.Net.Sockets;

namespace Weigelt.LastBoxStanding
{
	/// <summary>
	/// Helper class for monitoring a machine on the network.
	/// </summary>
	/// <remarks>
	/// Based on code by Peter A. Bromberg (pbromberg@yahoo.com,
	/// see http://www.eggheadcafe.com/articles/20020209.asp)
	/// which in turn is based on code by Lance Olson (MSDN Magazine 02/2001).
	/// </remarks>
	public class NetworkMachineMonitor : MachineMonitor
	{
		#region ... Helper class IcmpPacket ...

		/// <summary>
		///   Class that holds the Pack information
		/// </summary>
		private class IcmpPacket
		{
			/// <summary>
			/// Type of message.
			/// </summary>
			public byte Type;

			/// <summary>
			/// Type of sub code.
			/// </summary>
			public byte SubCode;

			/// <summary>
			/// Ones complement checksum of struct.
			/// </summary>
			public ushort Checksum;
			
			/// <summary>
			/// Identifier.
			/// </summary>
			public ushort Identifier;

			/// <summary>
			/// Sequence number.
			/// </summary>
			public ushort SequenceNumber;

			/// <summary>
			/// Byte array of data.
			/// </summary>
			public byte[] Data;
		}

		#endregion

		const int _SocketError = -1;
		const int _IcmpEcho = 8;
		const int _MaxPacketSize = 65535;
		const int _PingBytes=32;
		const int _MaxAttempts=4;
		const int _PingTimeout=5000; // 5000msec = 5sec

		#region ... Private helper methods ...

		/// <summary>
		///  This method creates an ICMP Packet
		/// </summary>
		private IcmpPacket CreateIcmpPacket(ushort identifier, ushort sequenceNumber, int pingBytes)
		{
			//
			// First create the packed object and initialize it as far as possible.
			//
			IcmpPacket objPingPacket=new IcmpPacket();
			objPingPacket.Type = _IcmpEcho; //8
			objPingPacket.SubCode = 0; 
			objPingPacket.Checksum = 0;
			objPingPacket.Identifier   = identifier;
			objPingPacket.SequenceNumber  = sequenceNumber;
			objPingPacket.Data = new Byte[pingBytes];
			//Initialize the Packet.Data
			for (int iCount = 0; iCount < pingBytes; iCount++)
			{
				objPingPacket.Data[iCount] = (byte)'#';
			}

			//
			// Then serialize it to a byte array.
			//
			int nPacketSize = pingBytes + 8;
			byte [] arrPacketBuffer = new byte[nPacketSize];

			Serialize(objPingPacket, arrPacketBuffer, nPacketSize, pingBytes );
			//
			// Calculate the checksum.
			//
			ushort [] arrChecksumBuffer = new ushort[Convert.ToInt32( Math.Ceiling( Convert.ToDouble(nPacketSize) / 2))];
			int nIcmpHeaderBufferIndex = 0;
			for( int nIndex = 0; nIndex < arrChecksumBuffer.Length; nIndex++ ) 
			{
				arrChecksumBuffer[nIndex] = BitConverter.ToUInt16(arrPacketBuffer, nIcmpHeaderBufferIndex);
				nIcmpHeaderBufferIndex += 2;
			}
			objPingPacket.Checksum = CalculateChecksum(arrChecksumBuffer);

			return objPingPacket;
		}
    
		/// <summary>
		///  This method is used to get the Packet and calculates the total size
		///  of the Pack by converting it to byte array
		/// </summary>
		private int Serialize(IcmpPacket thePacket, byte[] buffer, int packetSize, int pingData )
		{
			int nIndex = 0;

			// Serialize type
			byte [] arrTypeBytes = new byte[1];
			arrTypeBytes[0] = thePacket.Type;
			Array.Copy( arrTypeBytes, 0, buffer, nIndex, arrTypeBytes.Length );
			nIndex += arrTypeBytes.Length;

			// Serialize code
			byte [] arrCodeBytes = new byte[1];
			arrCodeBytes[0] = thePacket.SubCode;
			Array.Copy( arrCodeBytes, 0, buffer, nIndex, arrCodeBytes.Length );
			nIndex += arrCodeBytes.Length;

			// Serialize cksum
			byte [] arrChecksumBytes = BitConverter.GetBytes(thePacket.Checksum);
			Array.Copy( arrChecksumBytes, 0, buffer, nIndex, arrChecksumBytes.Length );
			nIndex += arrChecksumBytes.Length;

			// Serialize id
			byte [] arrIdentifierBytes    = BitConverter.GetBytes(thePacket.Identifier);
			Array.Copy( arrIdentifierBytes, 0, buffer, nIndex, arrIdentifierBytes.Length );
			nIndex += arrIdentifierBytes.Length;

			// Serialize sequence number
			byte [] arrSequenceNumberBytes   = BitConverter.GetBytes(thePacket.SequenceNumber);
			Array.Copy( arrSequenceNumberBytes, 0, buffer, nIndex, arrSequenceNumberBytes.Length );
			nIndex += arrSequenceNumberBytes.Length;

			// Copy the data   
			Array.Copy( thePacket.Data, 0, buffer, nIndex, pingData );
			nIndex += pingData;
			if( nIndex != packetSize/* sizeof(IcmpPacket)  */) 
				throw new InvalidOperationException("Error serializing IcmpPacket");

			return nIndex;
		}

		/// <summary>
		/// Calculates the checksum for the specified buffer.
		/// </summary>
		private ushort CalculateChecksum(ushort[] buffer)
		{
			int nChecksum = 0;
			for (int nIndex = 0; nIndex < buffer.Length; nIndex++) 
			{
				nChecksum += Convert.ToInt32( buffer[nIndex] );
			}

			nChecksum = (nChecksum >> 16) + (nChecksum & 0xFFFF);
			nChecksum += (nChecksum >> 16);
			return (ushort)((~nChecksum) & 0xFFFF);
		}
		#endregion

		//===================================================
		// Public
		//===================================================

		#region ... Public methods ...

		/// <summary>
		/// Determines whether the specified machine is running.
		/// </summary>
		/// <param name="machine">Name or IP address (as text) of the machine to check</param>
		public override bool IsMachineRunning(string machine)
		{
			bool bMachineSuccessfullyContacted=false;

			IPHostEntry objPingTarget=null;
			try
			{
				if (machine.IndexOf('.')!=-1)
					objPingTarget = Dns.GetHostByAddress(machine);
				else
					objPingTarget = Dns.GetHostByName(machine);
			}
			catch {}

			if (objPingTarget==null)
				return false;

			int nBytesReceived = 0;
			int nStartTime = 0;

			//Random ICMP identification.
			// This fixes the bug where this ICMP would listen to all ICMP transmits
			Random rndIdent = new Random();
			ushort iIdent = (ushort) rndIdent.Next(ushort.MaxValue);

			// Convert the server IP_EndPoint to an EndPoint
			IPEndPoint ipepServer = new IPEndPoint(objPingTarget.AddressList[0],0);
			EndPoint epServer = (ipepServer);

			// Set the receiving endpoint to the client machine
			IPHostEntry objPingSource = Dns.GetHostByName(Dns.GetHostName());
			IPEndPoint ipEndPointFrom = new IPEndPoint(objPingSource.AddressList[0],0);
			EndPoint   EndPointFrom = (ipEndPointFrom);

			ushort nSequenceNumber = 0;
			for (int nAttempt=0; ((nAttempt<_MaxAttempts) && (! bMachineSuccessfullyContacted)) ;nAttempt++)
			{
				Socket objPingSocket = new Socket(AddressFamily.InterNetwork, SocketType.Raw, ProtocolType.Icmp);
				//Set socket timeout, but this doesn't seem to work...
				objPingSocket.SetSocketOption( SocketOptionLevel.Socket, SocketOptionName.ReceiveTimeout, (int) _PingTimeout);
				objPingSocket.SetSocketOption( SocketOptionLevel.Socket, SocketOptionName.SendTimeout, (int) _PingTimeout);
        
				// Initialize the buffers. The receive buffer is the size of the
				// ICMP header plus the IP header (20 bytes)
				byte[] arrReceiveBuffer = new byte [_MaxPacketSize];

				nStartTime = System.Environment.TickCount; // Start timing

				//Create the next packet in our sequence
				IcmpPacket objPingPacket = CreateIcmpPacket(iIdent, nSequenceNumber++, _PingBytes);

				int nPacketSize = _PingBytes + 8;
				byte [] byteSendBuffer = new byte[ nPacketSize ];
				Serialize(objPingPacket, byteSendBuffer, nPacketSize, _PingBytes );

				//send the Packet over the socket
				int iResult = objPingSocket.SendTo(byteSendBuffer, nPacketSize, SocketFlags.None , epServer);
				if ((iResult) == _SocketError)
				{
					throw new InvalidOperationException("Socket error: Cannot send packet");
				}
				//Receive the bytes
				//loop while waiting checking the time of the server responding 

				while (true)
				{
					nBytesReceived = 0;
					try 
					{
						nBytesReceived = objPingSocket.ReceiveFrom(arrReceiveBuffer, _MaxPacketSize, SocketFlags.None, ref EndPointFrom);
					}
					catch
					{
						break;
					}
					//OK, check the ident & seq
					int iTemp = arrReceiveBuffer[25] * 256 + arrReceiveBuffer[24];
					ushort iRecIdent = (ushort) iTemp;
					iTemp = arrReceiveBuffer[27] * 256 + arrReceiveBuffer[26];
					ushort iRecSeq   = (ushort) iTemp;

					if ( (iRecIdent != iIdent) || (iRecSeq != (nSequenceNumber-1)) )   //Not our packet
					{   
						// Request timed out.
						break;
					}

					if (nBytesReceived == _SocketError)
					{
						// Host not Responding
						break;
					} 
					
					if (nBytesReceived > 0) 
					{
						if ((System.Environment.TickCount - nStartTime) > _PingTimeout) // timeout?
						{
							System.Diagnostics.EventLog.WriteEntry("PING", "Timeout");
							System.Threading.Thread.Sleep(System.TimeSpan.FromMilliseconds(1000));
							break;
						}

						//
						// If we get here, everything is Ok.
						//
						bMachineSuccessfullyContacted = true;
						break;
					} 
				} //while

				objPingSocket.Shutdown(SocketShutdown.Both);
				objPingSocket.Close();

				if ((! bMachineSuccessfullyContacted) && (nAttempt<_MaxAttempts-1))
					System.Threading.Thread.Sleep(System.TimeSpan.FromMilliseconds(1000));
			} // for
			return bMachineSuccessfullyContacted;
		}

		#endregion

		#region ... Constructor ...

		/// <summary>
		/// Creates a new <see cref="NetworkMachineMonitor"/> instance.
		/// </summary>
		public NetworkMachineMonitor()
		{
		}

		#endregion
	}
}
