#region ... Copyright  2005 Roland Weigelt ...
/*
This software is provided 'as-is', without any expressed or
implied warranty. In no event will the author be held liable
for any damages arising from the use of this software.

Permission is granted to anyone to use this software for any
purpose, including commercial applications, and to alter it and
redistribute it freely, subject to the following restrictions:

1.	The origin of this software must not be misrepresented; you
	must not claim that you wrote the original software. If you
	use this software in a product, an acknowledgment in the
	product documentation is required.
	
2.	Altered source versions must be plainly marked as such, and
	must not be misrepresented as being the original software.
	
3.	This notice may not be removed or altered from any source
	or altered source distribution.

Copyright  2005 Roland Weigelt (info@roland-weigelt.de)
*/
#endregion

using System;
using System.Collections.Specialized;
using System.Configuration;
using System.Xml;

namespace Weigelt.LastBoxStanding
{
	/// <summary>
	/// Configuration section handler for the service settings, e.g. the
	/// list of machines to monitor.
	/// </summary>
	public class ServiceConfigurationSectionHandler : IConfigurationSectionHandler
	{
		private sealed class Attr
		{
			private Attr() {}
			public const string Execute="execute";
			public const string Arguments="arguments";
			public const string Minutes="minutes";
			public const string Retries="retries";
			public const string SimulateActions="simulateActions";
			public const string Verbose="verbose";
		}

		private sealed class Tag
		{
			private Tag() {}
			public const string Actions="actions";
			public const string WhenLastRemaining="whenLastRemaining";
			public const string Options="options";
			public const string CheckInterval="checkInterval";
			public const string Diagnostics="diagnostics";
		}

		/// <summary>
		/// Returns the (integer) value of an attribute in the specified tag.
		/// </summary>
		/// <param name="parentTag">Parent Tag.</param>
		/// <param name="tagName">Name of the tag to read from.</param>
		/// <param name="attributeName">Name of the attribute.</param>
		/// <param name="defaultValue">Default value.</param>
		/// <returns>
		/// The value, or the specified default value if attribute is not found.
		/// </returns>
		private int ReadInteger(XmlElement parentTag, string tagName, string attributeName, int defaultValue)
		{
			XmlElement objTag=parentTag.SelectSingleNode(tagName) as XmlElement;
			if (objTag==null)
				return defaultValue;

			string strValue=objTag.GetAttribute(attributeName);
			if ((strValue==null) || (strValue.Length==0))
				return defaultValue;
			return Int32.Parse(strValue);
		}

		/// <summary>
		/// Returns the (boolean) value of an attribute in the specified tag.
		/// </summary>
		/// <param name="parentTag">Parent Tag.</param>
		/// <param name="tagName">Name of the tag to read from.</param>
		/// <param name="attributeName">Name of the attribute.</param>
		/// <param name="defaultValue">Default value.</param>
		/// <returns>
		/// The value, or the specified default value if attribute is not found.
		/// </returns>
		private bool ReadBoolean(XmlElement parentTag, string tagName, string attributeName, bool defaultValue)
		{
			XmlElement objTag=parentTag.SelectSingleNode(tagName) as XmlElement;
			if (objTag==null)
				return defaultValue;

			string strValue=objTag.GetAttribute(attributeName);
			if ((strValue==null) || (strValue.Length==0))
				return defaultValue;
			return Boolean.Parse(strValue);
		}

		/// <summary>
		/// Creates a <see cref="ActionConfiguration"/> by reading from the specified XML tag.
		/// </summary>
		/// <param name="parentTag">Parent Tag.</param>
		/// <param name="tagName">Name of the tag to read from.</param>
		/// <returns>
		/// The <see cref="ActionConfiguration"/>, or <c>null</c> if the specified tag does not exist.
		/// </returns>
		private ActionConfiguration ReadActionConfiguration(XmlElement parentTag, string tagName)
		{
			XmlElement objTag=parentTag.SelectSingleNode(tagName) as XmlElement;
			if (objTag==null)
				return null;
			
			ActionConfiguration objConfiguration=new ActionConfiguration(tagName);
			string strExecute=objTag.GetAttribute(Attr.Execute);
			if (strExecute!=null)
				objConfiguration.Executable=strExecute;

			string strArguments=objTag.GetAttribute(Attr.Arguments);
			if (strArguments!=null)
				objConfiguration.Arguments=strArguments;

			return objConfiguration;
		}

		//===================================================
		// Public
		//===================================================

		/// <summary>
		/// Creates the <see cref="StringCollection"/> containing the names of the
		/// machines to monitor.
		/// </summary>
		/// <param name="parent"></param>
		/// <param name="configContext"></param>
		/// <param name="section"></param>
		/// <returns></returns>
		public object Create(object parent, object configContext, XmlNode section)
		{
			ServiceConfiguration objConfiguration=new ServiceConfiguration();
			//
			// <monitoredMachines>
			//
			XmlNodeList objMachineNodes=section.SelectNodes("monitoredMachines/machine");
			string strName;
			foreach (XmlElement objMachineTag in objMachineNodes)
			{
				strName=objMachineTag.GetAttribute("name");
				if ((strName!=null) && (strName.Length>0))
					objConfiguration.MonitoredMachines.Add(strName);
			}
			//
			// <actions>
			//
			XmlElement objActionsTag=section.SelectSingleNode(Tag.Actions) as XmlElement;
			if (objActionsTag!=null)
			{
				objConfiguration.ActionWhenLastRemaining=ReadActionConfiguration(objActionsTag, Tag.WhenLastRemaining);
			}
			//
			// <options>
			//
			XmlElement objArgumentsTag=section.SelectSingleNode(Tag.Options) as XmlElement;
			if (objArgumentsTag!=null)
			{
				objConfiguration.CheckInterval=ReadInteger(objArgumentsTag, Tag.CheckInterval, Attr.Minutes, 10);
				objConfiguration.CheckRetries=ReadInteger(objArgumentsTag, Tag.CheckInterval, Attr.Retries, 2);
				objConfiguration.SimulateActions=ReadBoolean(objArgumentsTag, Tag.Diagnostics, Attr.SimulateActions, false);
				objConfiguration.Verbose=ReadBoolean(objArgumentsTag, Tag.Diagnostics, Attr.Verbose, false);
			}

			return objConfiguration;
		}
	}
}
