#region ... Copyright  2005 Roland Weigelt ...
/*
This software is provided 'as-is', without any expressed or
implied warranty. In no event will the author be held liable
for any damages arising from the use of this software.

Permission is granted to anyone to use this software for any
purpose, including commercial applications, and to alter it and
redistribute it freely, subject to the following restrictions:

1.	The origin of this software must not be misrepresented; you
	must not claim that you wrote the original software. If you
	use this software in a product, an acknowledgment in the
	product documentation is required.
	
2.	Altered source versions must be plainly marked as such, and
	must not be misrepresented as being the original software.
	
3.	This notice may not be removed or altered from any source
	or altered source distribution.

Copyright  2005 Roland Weigelt (info@roland-weigelt.de)
*/
#endregion

using System;
using System.Diagnostics;
using System.IO;
using System.ServiceProcess;

namespace Weigelt.LastBoxStanding
{
	/// <summary>
	/// Worker for the service, i.e. checks the configured machines.
	/// </summary>
	public class Worker
	{
		private readonly ServiceBase m_objService;
		private readonly MachineMonitor m_objMachineMonitor;

		private bool m_bAtLeastOneHasBeenRunning=false;
		private int m_nCurrentRetry=0;
		private ServiceConfiguration m_objConfiguration=new ServiceConfiguration();

		/// <summary>
		/// Gets the machine monitor used by this <see cref="Worker"/> instance.
		/// </summary>
		protected MachineMonitor MachineMonitor
		{
			get { return m_objMachineMonitor; }
		}

		#region ... Private helper methods ...

		/// <summary>
		/// Writes a message to the event log.
		/// </summary>
		/// <param name="type">Type of the event log entry.</param>
		/// <param name="message">Message of the event log entry.</param>
		private void WriteEventLogEntry(EventLogEntryType type, string message)
		{
			OnWriteEventLogEntry(type, message);
		}

		/// <summary>
		/// Writes a message to the event log.
		/// </summary>
		/// <param name="type">Type of the event log entry.</param>
		/// <param name="format">A <see cref="string"/> containing zero or more format items.</param>
		/// <param name="args">An <see cref="object"/> array  containing zero or more objects to format.</param>
		private void WriteEventLogEntry(EventLogEntryType type, string format, params object[] args)
		{
			WriteEventLogEntry(type, String.Format(format, args));
		}

		/// <summary>
		/// Performs the specified action.
		/// </summary>
		/// <param name="action"></param>
		private void PerformAction(ActionConfiguration action)
		{
			if (action==null)
				return;
			if ((action.Executable==null) || (action.Executable.Length==0))
			{
				if (m_objConfiguration.Verbose)
				{
					//
					// Note that we reported the missing configuration of the executable
					// when the service started, but since we're in "verbose" mode,
					// we'll report it each and every time.
					//
					WriteEventLogEntry(EventLogEntryType.Warning, "No executable specified in configuration for  \"{0}\" action.", action.Name);
				}
				return;
			}

			if (m_objConfiguration.SimulateActions)
			{
				OnSimulateAction(action);
				return;
			}

			try
			{
				if (m_objConfiguration.Verbose)
					WriteEventLogEntry(EventLogEntryType.Information, "Performing action \"{0}\".", action.Name);

				ProcessStartInfo objStartInfo=new ProcessStartInfo(action.Executable);
				if ((action.Arguments!=null) && (action.Arguments.Length>0))
					objStartInfo.Arguments=action.Arguments;

				objStartInfo.RedirectStandardError=true;
				objStartInfo.RedirectStandardOutput=true;
				objStartInfo.UseShellExecute=false;
				Process objProcess=Process.Start(objStartInfo);
				string strMessage=null;
				using (TextReader objStandardError=objProcess.StandardError)
				using (TextReader objStandardOutput=objProcess.StandardOutput)
				{
					objProcess.WaitForExit();
					if ((objProcess.ExitCode!=0) || (m_objConfiguration.Verbose))
					{
						string strErrorText=objStandardError.ReadToEnd();
						string strOutputText=objStandardOutput.ReadToEnd();

						if (strErrorText.Length>0)
							strMessage="Error text: "+strErrorText+Environment.NewLine;
						else
							strMessage=String.Empty;

						if (strOutputText.Length>0)
							strMessage="Output text: "+strOutputText+Environment.NewLine;
					}

					if (strMessage!=null)
					{
						if (objProcess.ExitCode!=0)
							WriteEventLogEntry(EventLogEntryType.Error,
								"Error while running {0} {1}:{2}{3}",
								action.Executable, action.Arguments,
								Environment.NewLine, strMessage);
						else
							WriteEventLogEntry(EventLogEntryType.Information,
								"Result of running {0} {1}:{2}{3}",
								action.Executable, action.Arguments,
								Environment.NewLine, strMessage);
					}
				}
			}
			catch (Exception exc)
			{
				WriteEventLogEntry(EventLogEntryType.Error, "Error executing {0} {1}:\n{2}", action.Executable, action.Arguments, exc.Message);
			}
		}

		#endregion

		#region ... Protected hook methods ...

		/// <summary>
		/// Called for writing an entry to the event log.
		/// </summary>
		/// <param name="type">Type of the event log entry.</param>
		/// <param name="message">Message of the event log entry.</param>
		protected virtual void OnWriteEventLogEntry(EventLogEntryType type, string message)
		{
			if (m_objService!=null)
				m_objService.EventLog.WriteEntry(message,type);
		}

		/// <summary>
		/// Called for simulating the specified action
		/// </summary>
		/// <param name="action">Action to be simulated.</param>
		protected virtual void OnSimulateAction(ActionConfiguration action)
		{
			WriteEventLogEntry(EventLogEntryType.Information, "Simulated action: {0} {1}", action.Executable, action.Arguments);
		}

		#endregion

		//===================================================
		// Public
		//===================================================

		/// <summary>
		/// Gets or sets the configuration of the worker.
		/// </summary>
		public ServiceConfiguration Configuration
		{
			get { return m_objConfiguration; }
			set { m_objConfiguration=value; }
		}

		/// <summary>
		/// Resets the worker.
		/// </summary>
		public void Reset()
		{
			m_bAtLeastOneHasBeenRunning=false;
		}

		/// <summary>
		/// Returns a value indicating whether at least one of the
		/// configured machines is still up and running.
		/// </summary>
		/// <returns></returns>
		public bool IsAtLeastOneMachineRunning()
		{
			foreach (string strMachine in m_objConfiguration.MonitoredMachines)
			{
				if (m_objMachineMonitor.IsMachineRunning(strMachine))
				{
					if (m_objConfiguration.Verbose)
						WriteEventLogEntry(EventLogEntryType.Information, "Machine {0} is running, skipping all remaining checks.", strMachine);
					return true; // at least one machine still running.
				}
				if (m_objConfiguration.Verbose)
					WriteEventLogEntry(EventLogEntryType.Information, "Machine {0} cannot be reached.", strMachine);
			}
			return false;
		}
		
		/// <summary>
		/// Performs the worker's actions; this method is called
		/// repeatedly by the service, each time after a certain interval.
		/// </summary>
		public void Run()
		{

			try
			{
				if (this.IsAtLeastOneMachineRunning())
				{
					m_bAtLeastOneHasBeenRunning=true;
					m_nCurrentRetry=0;
				}
				else
				{
					//
					// None of the monitored machines is running.
					//
					if (! m_bAtLeastOneHasBeenRunning)
					{
						//
						// This may be the case because the server was started first -- maybe
						// for maintenance, to it would be pretty dumb to shut it down immediately.
						//
						if (m_objConfiguration.Verbose)
							WriteEventLogEntry(EventLogEntryType.Information, "Since the service was started, none of the monitored machines could ever be reached.");
					}
					else
					{
						//
						// Ok, at least one of the monitored machines has been running in the past,
						// but couldn't be reached now. Maybe somebody simply performed a reboot
						// or disconnected the network for just a moment, so it's better to retry
						// before we'll really perform some action.
						//
						if (m_nCurrentRetry<m_objConfiguration.CheckRetries)
						{
							++m_nCurrentRetry;

							if (m_objConfiguration.Verbose)
								WriteEventLogEntry(EventLogEntryType.Information, "None of the monitored machines could be reached. Retry #{0} in {1} minute[s].", m_nCurrentRetry, m_objConfiguration.CheckInterval);
						}
						else
						{
							PerformAction(m_objConfiguration.ActionWhenLastRemaining);
							m_nCurrentRetry=0; // this may be important when the action is not configured to perform a shutdown.
						}
					}
				}
			}
			catch (Exception exc)
			{
				WriteEventLogEntry(EventLogEntryType.Error, "Unexpected error: {0}", exc.Message);
			}
			finally
			{
				//
				// Hmm... this shouldn't be necessary at all, but tests have shown that it decreases
				// the amount of memory used by the service AS SHOWN IN THE TASK MANAGER
				// (which I know is not necessarily the amount of memory actually used by the service).
				//
				System.GC.Collect();
			}
		}

		#region ... Constructors ...

		/// <summary>
		/// Creates a new <see cref="Worker"/> instance.
		/// </summary>
		/// <param name="service">The service that uses this worker.</param>
		/// <remarks>
		/// This constructor is used by the actual service.
		/// </remarks>
		public Worker(ServiceBase service)
		{
			m_objService=service;
			m_objMachineMonitor=new NetworkMachineMonitor();
			Reset();
		}

		/// <summary>
		/// Creates a new <see cref="Worker"/> instance.
		/// </summary>
		/// <param name="machineMonitor">The machine monitor to be used instead of the built-in.</param>
		/// <remarks>
		/// This constructor is only used for unit testing.
		/// </remarks>
		protected Worker(MachineMonitor machineMonitor)
		{
			m_objService=null;
			m_objMachineMonitor=machineMonitor;
			Reset();
		}

		#endregion
	}
}
