#region ... Copyright  2005 Roland Weigelt ...
/*
This software is provided 'as-is', without any expressed or
implied warranty. In no event will the author be held liable
for any damages arising from the use of this software.

Permission is granted to anyone to use this software for any
purpose, including commercial applications, and to alter it and
redistribute it freely, subject to the following restrictions:

1.	The origin of this software must not be misrepresented; you
	must not claim that you wrote the original software. If you
	use this software in a product, an acknowledgment in the
	product documentation is required.
	
2.	Altered source versions must be plainly marked as such, and
	must not be misrepresented as being the original software.
	
3.	This notice may not be removed or altered from any source
	or altered source distribution.

Copyright  2005 Roland Weigelt (info@roland-weigelt.de)
*/
#endregion

using System;
using System.Configuration;
using System.Diagnostics;
using System.Reflection;
using NUnit.Framework;

namespace Weigelt.LastBoxStanding.Test
{
	/// <summary>
	/// Tests the TestWorker class
	/// </summary>
	[ TestFixture ]
	public class WorkerTest
	{
		#region ... Helper class TestWorker ...

		private class TestWorker : Worker
		{
			private int m_nPerformedActionsCount=0;
			private string m_strLastActionPerformed=null;
			private EventLogEntryType m_eEventLogEntryType;
			private string m_strLastEventLogEntry=null;

			protected override void OnSimulateAction(ActionConfiguration action)
			{
				base.OnSimulateAction(action);

				m_strLastActionPerformed=String.Format("{0} {1}", action.Executable, action.Arguments);
				++m_nPerformedActionsCount;
			}

			protected override void OnWriteEventLogEntry(EventLogEntryType type, string message)
			{
				base.OnWriteEventLogEntry(type, message);
				m_eEventLogEntryType=type;
				m_strLastEventLogEntry=message;
				Console.WriteLine("[{0}] {1}", type, message);
			}

			public int PerformedActionsCount { get { return m_nPerformedActionsCount; } }
			public string LastActionPerformed { get { return m_strLastActionPerformed; } }
			public string LastEventLogEntry { get { return m_strLastEventLogEntry; } }
			public EventLogEntryType LastEventLogEntryType { get { return m_eEventLogEntryType; } }

			public new SimulatedMachineMonitor MachineMonitor
			{
				get { return base.MachineMonitor as SimulatedMachineMonitor; }
			}

			public TestWorker()
				:base(new SimulatedMachineMonitor())
			{
				this.Configuration.CheckInterval=10;
				this.Configuration.SimulateActions=true;
				this.Configuration.Verbose=true;
				this.Configuration.ActionWhenLastRemaining=new ActionConfiguration("theName", "theExecutable", "theArguments");
			}
		}
		#endregion

		#region ... Helpers ...

		private void WriteTestName()
		{
			StackFrame objStackFrame=new StackFrame(1);
			Console.WriteLine();
			Console.WriteLine(objStackFrame.GetMethod().Name);
		}
		#endregion

		[ Test ]
		public void AtLeastOneMachineRunning_NoAction()
		{
			WriteTestName();
			TestWorker objWorker=new TestWorker();
			objWorker.Configuration.MonitoredMachines.Add(SimulatedMachineMonitor.AlwaysOff);
			objWorker.Configuration.MonitoredMachines.Add(SimulatedMachineMonitor.AlwaysOn);
			objWorker.Configuration.CheckRetries=0;
			objWorker.Run();
			Assert.AreEqual(0, objWorker.PerformedActionsCount);
		}

		[ Test ]
		public void NoMachineEverRunning_NoAction()
		{
			WriteTestName();
			TestWorker objWorker=new TestWorker();
			objWorker.Configuration.MonitoredMachines.Add(SimulatedMachineMonitor.AlwaysOff);
			objWorker.Configuration.CheckRetries=0;
			objWorker.Run();
			Assert.AreEqual(0, objWorker.PerformedActionsCount);
		}

		[ Test ]
		public void RunningMachinesAreSwitchedOff_PerformAction()
		{
			WriteTestName();
			TestWorker objWorker=new TestWorker();
			objWorker.Configuration.MonitoredMachines.Add(SimulatedMachineMonitor.MachineA);
			objWorker.Configuration.MonitoredMachines.Add(SimulatedMachineMonitor.MachineB);
			objWorker.Configuration.CheckRetries=0;

			objWorker.MachineMonitor.MachineARunning=true;
			objWorker.MachineMonitor.MachineBRunning=true;
			objWorker.Run();
			Assert.AreEqual(0, objWorker.PerformedActionsCount, "No action expected");

			objWorker.MachineMonitor.MachineARunning=false;
			objWorker.MachineMonitor.MachineBRunning=false;
			objWorker.Run();
			Assert.AreEqual(1, objWorker.PerformedActionsCount, "Action expected");
			Assert.AreEqual("theExecutable theArguments", objWorker.LastActionPerformed, "(2) wrong action");
		}

		[ Test ]
		public void Retry_NoAction()
		{
			WriteTestName();
			TestWorker objWorker=new TestWorker();
			objWorker.Configuration.MonitoredMachines.Add(SimulatedMachineMonitor.MachineA);
			objWorker.Configuration.MonitoredMachines.Add(SimulatedMachineMonitor.OffAndOn);
			objWorker.Configuration.CheckRetries=1;
			//
			// First make sure that the worker knows that at least one machine has been running.
			//
			objWorker.MachineMonitor.MachineARunning=true;
			objWorker.Run();
			objWorker.MachineMonitor.MachineARunning=false;
			//
			// Now perform the actual test:
			//
			objWorker.Run();
			Assert.AreEqual(0, objWorker.PerformedActionsCount, "No action expected");

			objWorker.Run();
			Assert.AreEqual(0, objWorker.PerformedActionsCount, "No action expected");
		}

		[ Test ]
		public void Retry_PerformAction()
		{
			WriteTestName();
			TestWorker objWorker=new TestWorker();
			objWorker.Configuration.MonitoredMachines.Add(SimulatedMachineMonitor.MachineA);
			objWorker.Configuration.CheckRetries=2;
			//
			// First make sure that the worker knows that at least one machine has been running.
			//
			objWorker.MachineMonitor.MachineARunning=true;
			objWorker.Run();
			objWorker.MachineMonitor.MachineARunning=false;
			//
			// Now perform the actual test:
			//
			objWorker.Run();
			Assert.AreEqual(0, objWorker.PerformedActionsCount, "No action expected");

			objWorker.Run();
			Assert.AreEqual(0, objWorker.PerformedActionsCount, "No action expected");

			objWorker.Run();
			Assert.AreEqual(1, objWorker.PerformedActionsCount, "Action expected");
			Assert.AreEqual("theExecutable theArguments", objWorker.LastActionPerformed, "(2) wrong action");
		}
	}
}
